% from OLIVIER THUAL,
% http://thual.perso.enseeiht.fr/otapm/ond-surfli/index.htm
% modifi en mars 2012 par FY moulin
% permet le calcul de la propagation des ondes pour une condition initiale
% donne, par dcomposition dans l'espace spectrale, et recomposition
% ultrieure  tout instant t par somme des ondes planes moniochromatiques
% (dans cette version, on donne directement la condition initiale dans
% l'espace spectral)
function OndesEvolution()

% PARAMETRES PHYSIQUES ET NUMERIQUES
	L=1000.0;    % longueur du domaine (en m)
    gravite=9.81;   % acceleration de la pesanteur en ms-2
    profondeur=1.0; % profondeur d'eau H en metres
    lam=0.0000001;   % facteur d'amortissement des ondes (exp(-lam*k^2*t)
    flagondes=3;     % type d'ondes (1:surface libre, 2:ondes longues, 3:ondes longues avec rotation)
	fcor=0.0001;    % parametre de coriolis f (pour ondes de poincarr)
    N=2^10;     % Nombre de points pour le calcul
	dt=10.0;     % pas de temps du calcul (en s)
    M=20;       % nombre d'iterations temporelles
    um=0.2;     % echelle verticale pour affichage
    % -----------------------------------
    
%clear all
close all
%nomfilm='essai.avi';
%aviobj = avifile(nomfilm);
% flag=1 impulsion  2 paquet 
while 1 
    fprintf('1: impulsion vers la droite,  2: paquet d ondes vers la droite,\n'); 
    fprintf('3: impulsion (caillou dans l eau), 4: marche gaussienne,\n');
    fprintf('5: onde stationnaire, 6: ondes progressive\n');
	flag=input('7: marche droite (ajustement), 9: quit ?\n');
	close all
    if isempty(flag)
		flag=1
	end
	if flag==9
		return
    end
    
    ko=2*pi/L;
	dx=L/N;
	XA=0:dx:N/2*dx;
	XB=-N/2*dx:dx:-dx;
	KA=0:ko:(N/2-1)*ko;
	KB=-(N/2-1)*ko:ko:-ko;
	K=[KA 0 KB];
    
	% CHOIX DU TYPE DE CONDITION INITIALES
   % IMPULSION VERS LA DROITE
    if flag==1
		FZERO=[0*KA+1 0 0*KB];  % spectre blanc "plat" pour les ondes vers la droite
    end
   % PAQUET D'ONDE VERS LA DROITE
    if flag==2
		k0=50*ko; 
		k0=100*ko; 
		dk0=50*ko^2;
		FZERO=15*exp(-(K-k0).^2/dk0); % spectre gaussien centre en k0 pour les ondes vers la droite
    end
    
   % IMPULSION CENTRALE (CAILLOU DANS L'EAU) 
    if flag==3
		FZERO=[0*KA-1 0 0*KB-1]; % spectre blanc "plat" pour toutes les ondes (gauche et droite)
    end
   % IMPULSION CENTRALE GAUSSIENNE  
    if flag==4
		dk0=0.1*ko;
		FZERO=10*exp(-(K).^2/dk0); % spectre gaussien centre en 0 pour toutes les ondes
    end
  % ONDE STATIONNAIRE   
	if flag==5
		nnoeuds=4;
        FZERO=[0*KA 0 0*KB];
        FZERO(nnoeuds+1)=100.0;
        FZERO(N-nnoeuds+1)=100.0; % deux frquences non nulles pour une onde vers la droite et vers la gauche
	end
  % ONDE PROGRESSIVE VERS LA DROITE   
	if flag==6
		nnoeuds=1;
        FZERO=[0*KA 0 0*KB];
        FZERO(nnoeuds+1)=100.0; % 1 seule frequence non nulle, pour une onde vers la droite
    end
  % MARCHE D'EAU CENTRALE (AJUSTEMENT BARRAGE)  
	if flag==7
		dk0=2*ko;
		FZERO=10*sincAMOI(K/dk0); % spectre en sinc(x)=sin(x)/x pour la fonction heavyside en rel (marche)
        FZERO(1)=1.0;
    end
    
	% appel de la relation de dispersion water waves
	OMA=disper(KA,gravite,profondeur,flagondes,fcor);
	OMB=disper(KB,gravite,profondeur,flagondes,fcor);
	KD=[KB KA];
	OMEGA=[OMA 0 OMB];
	OMEG=[OMB OMA];

	%boucle sur le temps
	Mmax=M*dt;
	Mmaxh=Mmax/2;
	nla=0
	nlamax=21;
	for t=0:dt:Mmax
		t
		U=FZERO.*exp(-i*OMEGA*t).*exp(-lam*K.^2*t);
		U(N/2+1)=0;
		V=ifft(U,N);
		RV=real(V);
		YA=RV(1:N/2+1);
		YB=RV(N/2+1:N);
		X=[XB XA];
		Y=[YB YA];
		if t==0 
            figure('Position',[50 50 800 500]);
            plot(X,Y);
            axis([-L/2 L/2 -um um])
            xlabel('\it{x}','FontSize',30)
			ylabel('\eta','FontSize',30)
            title('condition initiale','FontSize',20)
            pause
        end
            figure('Position',[50 50 800 500]);
            plot(X,Y);
            axis([-L/2 L/2 -um um])
            xlabel('\it{x}','FontSize',30)
			ylabel('\eta','FontSize',30)
            title('evolution temporelle de la condition initiale','FontSize',20)
           nla=nla+1;
	end
end
%
% <<  RELATIONS DE DISPERION >>

% CHOIX DE LA RELATION DE DISPERSION
function om=disper(K,gravite,profondeur,flagondes,fcor)
if (flagondes==1) 
    om=disper_ondessurfacelibre(K,gravite,profondeur);
end
if (flagondes==2) 
    om=disper_ondeslongues(K,gravite,profondeur);
end
if (flagondes==3) 
    om=disper_ondespoincarre(K,gravite,profondeur,fcor);
end

% RELATION DE DISPERSION DES ONDES DE SURFACE (THEORIE POTENTIELLE)
function om=disper_ondessurfacelibre(K,gravite,profondeur)
om=sqrt(gravite*abs(K).*tanh(abs(K)*profondeur));

% RELATION DE DISPERSION DES ONDES LONGUES DE SURFACE
function om=disper_ondeslongues(K,gravite,profondeur)
om=sqrt(gravite*abs(K).*abs(K)*profondeur);

% RELATION DE DISPERSION DES ONDES LONGUES DE SURFACE AVEC ROTATION
function om=disper_ondespoincarre(K,gravite,profondeur,fcor)
om=sqrt(gravite*abs(K).*abs(K)*profondeur+fcor^2);

% << filew >>
%
% writing image on  files 
function filew(gcf,n)
[a,map]=capture(gcf);
filename=sprintf('image%2.2d.jpg',n)
imwrite(a,map,filename,'jpg')

function sortie=sincAMOI(x)
%sortie=1.0;
%if (abs(x) > 0.00001)
%    sortie=sin(x)/x;
%end
sortie=sin(x)./x;


