% logiciel dvelopp par Laurent LACAZE
% Resolution exacte des quations "shallow water", compltement non
% linaire

function SWexactLL()

clear all;
close all;
format long;

% PARAMETRES PHYSIQUES ET NUMERIQUES DE LA SIMULATION
g=9.81;     % acceleration de la pesanteur (ms^-2)
f=0.001;       % parametre de coriolis (rad s-1)
epsilonf=0.00001; % terme d'amortissement
L = 1000000;     % taille du domaine (m)
largeur=100000;  % largeur de la marche d'eau (m)
hl=2.25;     % hauteur d'eau  gauche et  droite de la marche (en m)
hr=2.75;     % hauteur d'eau sur la marche (en m)
Nx = 10000;  % nombre de points dans le domaine

%%%%%%% Euler method base on Kuganov model 2002, 2002 and 2007
%%%%%%% well-balanced method to reconstuct the source term numerical scheme
%%%%%%% dw/dt = -(Hj+1/2 - Hj-1/2)/dx + Sj (see Kuganov 2007 for H)
%%%%%%% Sj = -g/2dx * hmj+1/2[ (Bpj+1/2-Bpj-1/2) + (Zpj+1/2-Zpj-1/2) ]
%%%%%%%      -g/2dx * hpj-1/2[ (Bmj+1/2-Bmj-1/2) + (Zmj+1/2-Zmj-1/2) ]
%%%%%%% with B the topgraphy and Z the rheology such that Dz/Dx = Tau(z=0)/gh
%%%%%%% Euler in time!!


%% initialisation (rem : maillage regulier)


theta = 1;
% CONDITIONS INITIALES DU PROBLEME
ul=0;   
ur=0;
cl=0;
cr=0;
dul=0;
dur=0;
ulnm1=0;
urnm1=0;

x = linspace(-L,L,Nx);
dx = x(2)-x(1);
%epsilon = dx^4;
epsilon = 1.e-15.*dx^4;





u = zeros(size(x));
v = zeros(size(x));
h = zeros(size(x));
hu = zeros(size(x));
hv = zeros(size(x));
Cu = zeros(3,length(x)-2);
B = zeros(1,length(x)-1);

% CONDITION INITIALE MARCHE D'EAU
%h = hl*((x-largeur/2.0)<0)+ hr*(((x-largeur/2.0)>=0)&&((x+largeur/2.0)<=0)) + hl*((x+largeur/2.0)>0);
%h = hl*((x-largeur/2.0)<0)+ hr*((x-largeur/2.0)>=0)+hr*((x+largeur/2.0)<=0)+hl*((x+largeur/2.0)>0)
h = hr*((x+largeur/2.0)>=0)+hr*((x-largeur/2.0)<=0)-hr + hl*((x+largeur/2.0)<0)+hl*((x-largeur/2.0)>0);
%h = exp(-(x).^2)*0.01+hl;

   
    [ hmint,hpint,humint,hupint,hvmint,hvpint,umint,upint,vmint,vpint ] = VariablesBordMaille( theta,dx,h,u,v,hu,hv,epsilon );
    
    
    %%%
    
    [ apint, amint ] = AvancementCaracteristiques( g,hpint,hmint,upint,umint );
    %pause
%%


%% temporal loop
Nt = 130000;
t=0;

for n=1:Nt
    
    dtmax = dx/2./max(max([apint;-amint],[],1));
    dt = dtmax/10;
    
    %%% time evolution;
    
    [ RHSF, RHSS ] = RightHandSideMomentum(dx, amint,apint,umint,hmint,humint,hvmint,upint,hpint,hupint,hvpint,u,v,hu,hv,h,g,f,B,epsilonf );
    
        % time stepping
    Cu = [h(2:end-1); hu(2:end-1); hv(2:end-1)] + dt*(RHSF + RHSS);
    
    
    %%%% Boundary Conditions ??
    h(2:end-1) = Cu(1,:);
    %h(1) = hl;
    %h(end) = hr;
    h(1)=h(2);
    h(end)=h(end-1);
    
     hu(2:end-1) = Cu(2,:);
    u = sqrt(2).*h.*(hu)./sqrt(h.^4+max([h.^4;epsilon*ones(size(h))],[],1));
    
     %u(1) = ul + dt./dx.*cl*(dul);
     %u(end) = ur - dt./dx.*cr*(dur);
     u(1)=0;
     u(end)=0;
     
     hu(1) = h(1)*u(1);

     hu(end) = h(end)*u(end);
    
    hv(2:end-1) = Cu(3,:);
    v = sqrt(2).*h.*(hv)./sqrt(h.^4+max([h.^4;epsilon*ones(size(h))],[],1));
    hv(1) = hv(2);
    hv(end) = hv(end-1);
    v(1)=hv(1)/h(1);
    
    %%% 
    
    
    
    [ hmint,hpint,humint,hupint,hvmint,hvpint,umint,upint,vmint,vpint ] = VariablesBordMaille( theta,dx,h,u,v,hu,hv,epsilon );
    
    
%     c1=dx/dt.*(u(1)-u1)./(u(2)-u(1))
%     cend =-dx/dt.*(u(end)-uend)./(u(end)-u(end-1))
%     
%     if (abs(u(2)-u(1))<1.e-15)
%         c1=0;
%     end
%     if (abs(u(end)-u(end-1))<1.e-15)
%         cend=0;
%     end
%     
%     u1=u(1);
%     u2=u(2);
%     uend=u(end);
%     uendm1=u(end-1);
    
    
    
    
    
    %%%
    
    [ apint, amint ] = AvancementCaracteristiques( g,hpint,hmint,upint,umint );
    
    
    
    ul=u(1);
    cl=sqrt(g*h(1));
    cl=dx/dt.*(u(1)-ulnm1)./(u(2)-u(1));
    if abs(u(1)-ulnm1)<1.e-15
       cl=0;
    end
    
    dul=u(2)-u(1);
    
    ur=u(end);
    cr=sqrt(g*h(end));
    cr=-dx/dt.*(u(end)-urnm1)./(u(end)-u(end-1));
    if abs(u(end)-urnm1)<1.e-15
       cr=0;
    end
    
    dur=u(end)-u(1);
    
    ulnm1=u(1);
    urnm1=u(end);
    
    %%%%% POST PRO
    
    
      if floor((n-1)/100)==(n-1)/100
      
    figure(1)
    plot(x,h,'.')
    axis([-L,L,0,4])
    title(['Time: ', int2str(t), ' seconds']);
      ylabel('Height (m)');
      xlabel('Distance (m)');
    figure(2)
    plot(x,u,'.')
    axis([-L,L,-0.5,0.5]);
    title(['Time: ', int2str(t), ' seconds']);
      ylabel('U Speed (m/s)');
      xlabel('Distance (m)');
%    hold on
    %plot(x(2:end-1),fint,'r')
    %plot(x(2:end-1),RHSF(2,:),'g')
%   hold off
    figure(3)
    plot(x,v,'.')
    axis([-L,L,-0.5,0.5])
    title(['Time: ', int2str(t), ' seconds']);
      ylabel('V Speed (m/s)');
      xlabel('Distance (m)');
%    figure(4)
%    plot(n,sum(h.^2),'.')
%    hold on
    if (n==1)
        pause
    end
    n
    pause(0.01)
    end
    
t = t+dt;    
end
%%

