%%function GEOADJ is a re-write of a program written by Durran (1998)
%	   which numerically solves the Geostrophic Adjustment problem
%	   described in chapter 7.2 of Gill (1982).
%	   (More later...)
%last updated 14mar05 lmh
% -------------------------------------------
% FONCTION REPRISE PAR F. MOULIN avril 2013
% rsoud les quations "shallow water" LINEARISEES !!!
% condition initiale : "marche" d'eau en cartsien

function SWlinearise()

clear
clf

%%%Constants -- mostly user defined
nv = 3;	   %number of 1-d functions used in FS of soln (check this ---lmh)
nt = 3;	   %number of time levels at which each function is defined

% PARAMETRES DE LA SIMULATION
nx = 10000;          %    nombre de points en x
xmin = 0;           % borne gauche du domaine en x (en m)
xmax = 1.0e6;       % borne droite du domaine en x (en m)
g = 9.81;           % acceleration de la pesanteur
fcor = .001;      % parametre de coriolis
H=2.5;              % profondeur moyenne d'eau
dt=10;            % pas de temps
cspd = sqrt(H*g);   % vitesse de phase des ondes longues
steps = 20000;      % nombre de pas de temps
dfreq = 100;	    % frquence d'affichage des profils

% REGLAGE DE LA FENETRE D'AFFICHAGE
winl = round(0.45*nx); % borne gauche en indice
winr = round(0.55*nx); % borne droite en indice

dx = (xmax-xmin)/(nx-1);
dwidth = xmax - xmin; 
if (dt>(dx/cspd))
    fprintf('pas de temps trop long, instabilite');
    pause
end

imgno = 1;            %Image number counter
ismooth = 3;          %Number of smoothing passes to make *on IC*
isprint = 1;          %Whether or not to output images


%enum-type things
u = 1;
v = 2;
eta = 3;

%%%Computed data

%x-coordinates
for i=1:nx
    xx(i) = (i-1)*dx;
end

%%%Running data
f = zeros(nx,nv,nt);	 %solution mtx; format is:
			 % first: x-direction
			 % second: variable (u,v, and eta)
			 % third: timestep


%%%Initial conditions

%u,v = 0
f(:,u:v,1:2) = zeros(nx,2,2);

%start height with a step fn; using arctan approximation
%      with center at the half point and a scaling factor
%      of .02*domain width (i.e. .02*(xmax-xmin))
%f(:,eta,1) =  atan( (xx - dwidth/2)/(.01*dwidth));
%f(:,eta,1) = H*0.999*( (xx >= dwidth/2)*2 - 1 );
% CREATION MARCHE D'EAU
deltax=dwidth/100;
f(:,eta,1) = H*0.2*(( (xx >= (dwidth/2-deltax))*2 - 1 ) + ( (xx <= (dwidth/2+deltax))*2 - 1 ));
%f(:,eta,1) = H*0.25*(( (xx >= (dwidth/2-deltax))*2 - 1 )+ (xx <= (dwidth/2+deltax))*2 - 1  ));



%%Smoother for IC (not Laplacian and *definitely* not Lagrangian)
for i = 1:ismooth
  f(2:nx-1,eta,1) = .25*( f(3:nx,eta,1) + f(1:nx-2,eta,1) +...
			    2*f(2:nx-1,eta,1) );
  %%smooth endpoints
  f(1,eta,1) = f(2,eta,1);
  f(nx,eta,1) = f(nx-1,eta,1);
end


%%%Main integrator loop
%forward backward with 2nd order diffusion

%note that we are discarding the old times after two
%timesteps to save memory
for ts = 0:steps
    
    t = ts*dt; %actual time
    tnew = 2;
    tmid = 1;

    %calculate variable BC speed
    if (t >= (dwidth)/2/cspd)
      cstar = (dwidth)/2/t;
    else
      cstar = cspd;
    end
    %cstar=cspd;

    %smoothing factor
    if (ts <= 2)
      smoov = 1/4;
    else
      smoov = 1/1000;
    end
    %smoov=0;
    
    if mod(ts,dfreq) == 0

      left = xx(winl);
      right = xx(winr);
      len = xx(winl:winr);

            subplot(3,1,u);
      plot(len, f(winl:winr,u,tmid),...
	    'MarkerSize', 5);
      axis([left right -2 2]);
      title(['Time: ', int2str(t), ' seconds'])
      ylabel('U Speed (m/s)')
      
      %v
      subplot(3,1,v);
      plot(len, f(winl:winr,v,tmid),...
	    'EraseMode', 'xor', 'MarkerSize', 5);
      axis([left right -2 3]);
      ylabel('V Speed (m/s)')
      
      %height
      subplot(3,1,eta);
      plot(len, f(winl:winr,eta,tmid),...
	    'EraseMode', 'xor', 'MarkerSize', 5);
      axis([left right -H H]);
      ylabel('Height (m)');
      xlabel('Distance (m)');
      
      if (isprint) 
	%print('-dpng', '-noui', '-r100', sprintf('imgs/%03d', imgno));
	imgno = imgno + 1;
      end
      
      if (ts==0)
        pause
      end
  
      pause(.01)
    end
    
    %u velocity integration;
    f(2:nx-1,u,tnew) = f(2:nx-1,u,tmid) + dt*( fcor*f(2:nx-1,v,tmid) - ...
         g/dx*(f(2:nx-1,eta,tmid) - f(1:nx-2,eta,tmid) ) ) + ...
	smoov*(f(3:nx,u,tmid)-2*f(2:nx-1,u,tmid)+f(1:nx-2,u,tmid)) ;
    %let the bdy u-velocity be the same as the first calculated point
    f(1,u,tnew) = f(2,u,tnew);
    f(nx,u,tnew) = f(nx,u,tmid) + dt*(fcor*f(nx,v,tmid) - ...
	 g/dx*(f(nx,eta,tmid) - f(nx-1,eta,tmid) ) );

    %v velocity integration
    f(:,v,tnew) = f(:,v,tmid) - dt*fcor*f(:,u,tnew);

    %height integration
    f(2:nx-1,eta,tnew) =  f(2:nx-1,eta,tmid) - ...
	dt*H/dx*(f(3:nx,u,tnew) - f(2:nx-1,u,tnew) ) + ...
	smoov*(f(3:nx,eta,tnew)-2*f(2:nx-1,eta,tnew)+f(1:nx-2,eta,tnew)) ;
    %bdy heights
    f(nx,eta,tnew) = f(nx,eta,tmid) - dt*cstar/dx*...
	 (f(nx,eta,tmid) - f(nx-1,eta,tmid));
    f(1,eta,tnew) = f(1,eta,tmid) + dt*cstar/dx*...
	 (f(2,eta,tmid) - f(1,eta,tmid));

    

    f(:,:,tmid) = f(:,:,tnew);
	
end   %end integration